@@ -22,6 +22,7 @@ HTTP requests).
 
 import base64
 import collections
+import logging
 from datetime import datetime
 from datetime import timedelta
 import functools
@@ -31,6 +32,8 @@ import re
 import requests
 import random
 import time
+import math
+import sys
 
 import googlemaps
 
@@ -39,6 +42,8 @@ try: # Python 3
 except ImportError: # Python 2
     from urllib import urlencode
 
+logger = logging.getLogger(__name__)
+
 _X_GOOG_MAPS_EXPERIENCE_ID = "X-Goog-Maps-Experience-ID"
 _USER_AGENT = "GoogleGeoApiClientPython/%s" % googlemaps.__version__
 _DEFAULT_BASE_URL = "https://maps.googleapis.com"
@@ -52,8 +57,9 @@ class Client:
     def __init__(self, key=None, client_id=None, client_secret=None,
                  timeout=None, connect_timeout=None, read_timeout=None,
                  retry_timeout=60, requests_kwargs=None,
-                 queries_per_second=50, channel=None,
+                 queries_per_second=60, queries_per_minute=6000,channel=None,
                  retry_over_query_limit=True, experience_id=None, 
+                 requests_session=None,
                  base_url=_DEFAULT_BASE_URL):
         """
         :param key: Maps API key. Required, unless "client_id" and
@@ -92,11 +98,16 @@ class Client:
             seconds.
         :type retry_timeout: int
 
-        :param queries_per_second: Number of queries per second permitted.
+        :param queries_per_second: Number of queries per second permitted. Unset queries_per_minute to None. If set smaller number will be used.
             If the rate limit is reached, the client will sleep for the
             appropriate amount of time before it runs the current query.
         :type queries_per_second: int
 
+        :param queries_per_minute: Number of queries per minute permitted. Unset queries_per_second to None. If set smaller number will be used.
+            If the rate limit is reached, the client will sleep for the
+            appropriate amount of time before it runs the current query.
+        :type queries_per_minute: int
+
         :param retry_over_query_limit: If True, requests that result in a
             response indicating the query rate limit was exceeded will be
             retried. Defaults to True.
@@ -116,6 +127,9 @@ class Client:
             implemented. See the official requests docs for more info:
             http://docs.python-requests.org/en/latest/api/#main-interface
         :type requests_kwargs: dict
+
+        :param requests_session: Reused persistent session for flexibility.
+        :type requests_session: requests.Session
         
         :param base_url: The base URL for all requests. Defaults to the Maps API
             server. Should not have a trailing slash.
@@ -136,7 +150,7 @@ class Client:
                     "and hyphen (-) characters are allowed. If used without "
                     "client_id, it must be 0-999.")
 
-        self.session = requests.Session()
+        self.session = requests_session or requests.Session()
         self.key = key
 
         if timeout and (connect_timeout or read_timeout):
@@ -165,10 +179,26 @@ class Client:
             "timeout": self.timeout,
             "verify": True,  # NOTE(cbro): verify SSL certs.
         })
-
+        
+        self.queries_quota : int
         self.queries_per_second = queries_per_second
+        self.queries_per_minute = queries_per_minute
+        try: 
+            if (type(self.queries_per_second) == int and type(self.queries_per_minute) == int ):
+                self.queries_quota =  math.floor(min(self.queries_per_second, self.queries_per_minute/60))
+            elif (self.queries_per_second and type(self.queries_per_second) == int ):
+                self.queries_quota = math.floor(self.queries_per_second)
+            elif (self.queries_per_minute and type(self.queries_per_minute) == int ):
+                self.queries_quota = math.floor(self.queries_per_minute/60)
+            else:
+                sys.exit("MISSING VALID NUMBER for queries_per_second or queries_per_minute")
+            logger.info("API queries_quota:", self.queries_quota)
+
+        except NameError:
+            sys.exit("MISSING VALUE for queries_per_second or queries_per_minute")
+
         self.retry_over_query_limit = retry_over_query_limit
-        self.sent_times = collections.deque("", queries_per_second)
+        self.sent_times = collections.deque("", self.queries_quota)
         self.set_experience_id(experience_id)
         self.base_url = base_url
 
@@ -299,7 +329,7 @@ class Client:
         # Check if the time of the nth previous query (where n is
         # queries_per_second) is under a second ago - if so, sleep for
         # the difference.
-        if self.sent_times and len(self.sent_times) == self.queries_per_second:
+        if self.sent_times and len(self.sent_times) == self.queries_quota:
             elapsed_since_earliest = time.time() - self.sent_times[0]
             if elapsed_since_earliest < 1:
                 time.sleep(1 - elapsed_since_earliest)
@@ -398,7 +428,7 @@ from googlemaps.places import places_photo
 from googlemaps.places import places_autocomplete
 from googlemaps.places import places_autocomplete_query
 from googlemaps.maps import static_map
-
+from googlemaps.addressvalidation import addressvalidation
 
 def make_api_method(func):
     """
@@ -442,6 +472,7 @@ Client.places_photo = make_api_method(places_photo)
 Client.places_autocomplete = make_api_method(places_autocomplete)
 Client.places_autocomplete_query = make_api_method(places_autocomplete_query)
 Client.static_map = make_api_method(static_map)
+Client.addressvalidation = make_api_method(addressvalidation)
 
 
 def sign_hmac(secret, payload):
