@@ -21,9 +21,7 @@ import warnings
 from googlemaps import convert
 
 
-PLACES_FIND_FIELDS_BASIC = set(
-    [
-        "business_status",
+PLACES_FIND_FIELDS_BASIC = {"business_status",
         "formatted_address",
         "geometry",
         "geometry/location",
@@ -42,13 +40,11 @@ PLACES_FIND_FIELDS_BASIC = set(
         "photos",
         "place_id",
         "plus_code",
-        "types",
-    ]
-)
+        "types",}
 
-PLACES_FIND_FIELDS_CONTACT = set(["opening_hours"])
+PLACES_FIND_FIELDS_CONTACT = {"opening_hours"}
 
-PLACES_FIND_FIELDS_ATMOSPHERE = set(["price_level", "rating", "user_ratings_total"])
+PLACES_FIND_FIELDS_ATMOSPHERE = {"price_level", "rating", "user_ratings_total"}
 
 PLACES_FIND_FIELDS = (
     PLACES_FIND_FIELDS_BASIC
@@ -56,9 +52,7 @@ PLACES_FIND_FIELDS = (
     ^ PLACES_FIND_FIELDS_ATMOSPHERE
 )
 
-PLACES_DETAIL_FIELDS_BASIC = set(
-    [
-        "address_component",
+PLACES_DETAIL_FIELDS_BASIC = {"address_component",
         "adr_address",
         "business_status",
         "formatted_address",
@@ -82,17 +76,11 @@ PLACES_DETAIL_FIELDS_BASIC = set(
         "type",
         "url",
         "utc_offset",
-        "vicinity",
-    ]
-)
+        "vicinity",}
 
-PLACES_DETAIL_FIELDS_CONTACT = set(
-    ["formatted_phone_number", "international_phone_number", "opening_hours", "website"]
-)
+PLACES_DETAIL_FIELDS_CONTACT = {"formatted_phone_number", "international_phone_number", "opening_hours", "website"}
 
-PLACES_DETAIL_FIELDS_ATMOSPHERE = set(
-    ["price_level", "rating", "review", "user_ratings_total"]
-)
+PLACES_DETAIL_FIELDS_ATMOSPHERE = {"price_level", "rating", "review", "user_ratings_total"}
 
 PLACES_DETAIL_FIELDS = (
     PLACES_DETAIL_FIELDS_BASIC
@@ -100,6 +88,13 @@ PLACES_DETAIL_FIELDS = (
     ^ PLACES_DETAIL_FIELDS_ATMOSPHERE
 )
 
+DEPRECATED_FIELDS = {"permanently_closed"}
+DEPRECATED_FIELDS_MESSAGE = (
+    "Fields, %s, are deprecated. "
+    "Read more at https://developers.google.com/maps/deprecations."
+)
+
+
 def find_place(
     client, input, input_type, fields=None, location_bias=None, language=None
 ):
@@ -116,10 +111,9 @@ def find_place(
                   or 'phonenumber'.
     :type input_type: string
 
-    :param fields: The fields specifying the types of place data to return,
-                   separated by a comma. For full details see:
+    :param fields: The fields specifying the types of place data to return. For full details see:
                    https://developers.google.com/places/web-service/search#FindPlaceRequests
-    :type input: list
+    :type fields: list
 
     :param location_bias: Prefer results in a specified area, by specifying
                           either a radius plus lat/lng, or two lat/lng pairs
@@ -143,7 +137,14 @@ def find_place(
             "the given value is invalid: '%s'" % input_type
         )
 
-    if fields:       
+    if fields:
+        deprecated_fields = set(fields) & DEPRECATED_FIELDS
+        if deprecated_fields:
+            warnings.warn(
+                DEPRECATED_FIELDS_MESSAGE % str(list(deprecated_fields)),
+                DeprecationWarning,
+            )
+
         invalid_fields = set(fields) - PLACES_FIND_FIELDS
         if invalid_fields:
             raise ValueError(
@@ -167,7 +168,7 @@ def find_place(
 
 def places(
     client,
-    query,
+    query=None,
     location=None,
     radius=None,
     language=None,
@@ -422,6 +423,13 @@ def place(client, place_id, session_token=None, fields=None, language=None):
     params = {"placeid": place_id}
 
     if fields:
+        deprecated_fields = set(fields) & DEPRECATED_FIELDS
+        if deprecated_fields:
+            warnings.warn(
+                DEPRECATED_FIELDS_MESSAGE % str(list(deprecated_fields)),
+                DeprecationWarning,
+            )
+
         invalid_fields = set(fields) - PLACES_DETAIL_FIELDS
         if invalid_fields:
             raise ValueError(
@@ -457,13 +465,13 @@ def places_photo(client, photo_reference, max_width=None, max_height=None):
     :rtype: iterator containing the raw image data, which typically can be
         used to save an image file locally. For example:
 
-        ```
+    .. code-block:: python
+
         f = open(local_filename, 'wb')
         for chunk in client.places_photo(photo_reference, max_width=100):
             if chunk:
                 f.write(chunk)
         f.close()
-        ```
     """
 
     if not (max_width or max_height):
@@ -493,6 +501,7 @@ def places_autocomplete(
     input_text,
     session_token=None,
     offset=None,
+    origin=None,
     location=None,
     radius=None,
     language=None,
@@ -517,6 +526,12 @@ def places_autocomplete(
                    service will match on 'Goo'.
     :type offset: int
 
+    :param origin: The origin point from which to calculate straight-line distance
+                    to the destination (returned as distance_meters).
+                    If this value is omitted, straight-line distance will
+                    not be returned.
+    :type origin: string, dict, list, or tuple
+
     :param location: The latitude/longitude value for which you wish to obtain the
                      closest, human-readable address.
     :type location: string, dict, list, or tuple
@@ -550,6 +565,7 @@ def places_autocomplete(
         input_text,
         session_token=session_token,
         offset=offset,
+        origin=origin,
         location=location,
         radius=radius,
         language=language,
@@ -603,6 +619,7 @@ def _autocomplete(
     input_text,
     session_token=None,
     offset=None,
+    origin=None,
     location=None,
     radius=None,
     language=None,
@@ -621,6 +638,8 @@ def _autocomplete(
         params["sessiontoken"] = session_token
     if offset:
         params["offset"] = offset
+    if origin:
+        params["origin"] = convert.latlng(origin)
     if location:
         params["location"] = convert.latlng(location)
     if radius:
@@ -638,3 +657,4 @@ def _autocomplete(
 
     url = "/maps/api/place/%sautocomplete/json" % url_part
     return client._request(url, params).get("predictions", [])
+    
\ No newline at end of file
